------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2005                 www.xilinx.com
--
--  XAPP 486 - 7:1 LVDS in Spartan3E Devices
--
------------------------------------------------------------------------------
--
--  File name :      serdes_4b_7to1_fifo.vhd
--
--  Description :    generic 7:1 serdes for Spartan 3E with input fifo
-- 			data is transmitted LSBs first
-- 			0, 4,  8, 12, 16, 20, 24
-- 			1, 5,  9, 13, 17, 21, 25
-- 			2, 6, 10, 14, 18, 22, 26
-- 			3, 7, 11, 15, 19, 23, 27
--
--			The depth of the FIFO is selectable for either 32 or 1024 deep by
-- 			using comments. See the code around line 270.
--
--  Date - revision : November 29th 2006 - v 1.2
--
--  Author :         NJS
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regardg use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstandg the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2005 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------	
-- 
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity serdes_4b_7to1_fifo is port (
	clk 			: in std_logic ;			-- clock
	clkx3p5 		: in std_logic ;			-- 3.5 times clock
	clkx3p5not		: in std_logic ;			-- not 3.5 times clock
	datain 			: in std_logic_vector(27 downto 0) ;	-- input data
	rst 			: in std_logic ;			-- rst
	dataout			: out std_logic_vector(7 downto 0) ;	-- output data
	clkout			: out std_logic_vector(1 downto 0)) ;	-- output clock (1x)
end serdes_4b_7to1_fifo;


architecture arch_serdes_4b_7to1_fifo of serdes_4b_7to1_fifo is

signal low        	: std_logic;
signal high       	: std_logic;
signal syncp      	: std_logic;
signal ce0        	: std_logic;
signal ce1        	: std_logic;
signal ce1_d       	: std_logic;
signal db         	: std_logic_vector(15 downto 0);
signal pbp        	: std_logic;
signal pbp_d      	: std_logic;
signal dataind    	: std_logic_vector(27 downto 0);
signal ramdatain  	: std_logic_vector(31 downto 0);
signal ramdataout 	: std_logic_vector(15 downto 0);
signal ram_raddr  	: std_logic_vector(9 downto 0);
signal ram_waddr  	: std_logic_vector(8 downto 0);
signal ramoe        	: std_logic;
signal notramoe        	: std_logic;
signal ramwe        	: std_logic;
signal raddr	  	: std_logic_vector(9 downto 0);
signal raddr_d	  	: std_logic_vector(9 downto 0);
signal waddr	  	: std_logic_vector(8 downto 0);
signal shifterp     	: std_logic_vector(6 downto 0);
signal shifterp_d     	: std_logic_vector(6 downto 0);
signal dint     	: std_logic_vector(7 downto 0);
signal shifterp_pre    	: std_logic;
signal shifterp_pre1    : std_logic;
signal shifterp_pre2   	: std_logic;
signal shifterp_pre_d   : std_logic;
signal shifterp_pre1_d  : std_logic;
signal shifterp_pre2_d  : std_logic;
signal race	  	: std_logic;
signal race_d  		: std_logic;
signal race2	  	: std_logic;
signal race2_d  	: std_logic;
signal cint  		: std_logic;

-- relative placement information

attribute init     	: string;
attribute rloc     	: string;
attribute bel     	: string;

attribute rloc of fd_db0	: label is "x0y0";
attribute rloc of fd_db1	: label is "x0y0";
attribute rloc of fd_db2	: label is "x0y1";
attribute rloc of fd_db3	: label is "x0y1";
attribute rloc of fd_db4	: label is "x0y2";
attribute rloc of fd_db5	: label is "x0y2";
attribute rloc of fd_db6	: label is "x0y3";
attribute rloc of fd_db7	: label is "x0y3";

attribute rloc of fd_db8	: label is "x0y4";
attribute rloc of fd_db9	: label is "x0y4";
attribute rloc of fd_db10	: label is "x0y5";
attribute rloc of fd_db11	: label is "x0y5";
attribute rloc of fd_db12	: label is "x0y6";
attribute rloc of fd_db13	: label is "x0y6";
attribute rloc of fd_db14	: label is "x0y7";
attribute rloc of fd_db15	: label is "x0y7";

attribute rloc of muxp0   	: label is "x3y2";
attribute rloc of muxp1   	: label is "x3y2";
attribute rloc of muxp2   	: label is "x3y3";
attribute rloc of muxp3   	: label is "x3y3";
attribute rloc of muxp4   	: label is "x3y4";
attribute rloc of muxp5   	: label is "x3y4";
attribute rloc of muxp6   	: label is "x3y5";
attribute rloc of muxp7   	: label is "x3y5";

attribute rloc of fd_pbp0 	: label is "x3y7";
attribute rloc of fd_sp   	: label is "x3y7";
attribute rloc of fd_ce1   	: label is "x3y6";
attribute bel  of fd_ce1   	: label is "FFX";

attribute rloc of fd_rad0  	: label is "x2y5";
attribute rloc of fd_rad1  	: label is "x2y6";
attribute rloc of fd_rad2  	: label is "x2y6";
attribute rloc of fd_rad3  	: label is "x2y7";
attribute rloc of fd_rad4  	: label is "x2y2";
attribute rloc of fd_rad5  	: label is "x2y2";
attribute rloc of fd_rad6  	: label is "x2y1";
attribute rloc of fd_rad7  	: label is "x2y1";
attribute rloc of fd_rad8  	: label is "x3y0";
attribute rloc of fd_rad9  	: label is "x3y1";

attribute rloc of fd_race 	: label is "x1y6";
attribute rloc of fd_race2 	: label is "x1y7";

attribute rloc of fd_ramoe 	: label is "x1y6";

attribute rloc of fd_shpre  	: label is "x1y0";
attribute rloc of fd_shpre1 	: label is "x1y0";
attribute rloc of fd_shpre2 	: label is "x1y1";

attribute rloc of fd_shp0	: label is "x1y2";
attribute rloc of fd_shp1	: label is "x1y2";
attribute rloc of fd_shp2	: label is "x1y3";
attribute rloc of fd_shp3	: label is "x1y3";
attribute rloc of fd_shp4	: label is "x1y4";
attribute rloc of fd_shp5	: label is "x1y4";
attribute rloc of fd_shp6	: label is "x1y5";

attribute bel  of fd_ca		: label is "FFX";
attribute rloc of fd_ca		: label is "x2y3";
attribute rloc of fd_cb		: label is "x2y4";

begin

low      <= '0';
high     <= '1';
notramoe <= not ramoe ;

process (clk, rst)					-- ram write counter, disables write every 8 clocks
begin
if rst = '1' then
    	waddr <= "000000000";
    	ramwe <= '0';
elsif clk'event and clk = '1' then
    	waddr <= waddr + 1;
    	if waddr(2 downto 0) = "110" then
      		ramwe <= '0';
    	else
      		ramwe <= '1';
    	end if ;
end if;
end process;

fd_rad0 	: fdce  port map (c => clkx3p5, d => raddr_d(0), clr => rst, ce => ramoe, q => raddr(0));	-- ram read counter

race_d <= '1' when ramoe = '0' and raddr(0) = '1' else '0' ;
fd_race		: fdc   port map (c => clkx3p5, d => race_d,     clr => rst,  q => race);	
	
fd_rad1 	: fdce  port map (c => clkx3p5, d => raddr_d(1), clr => rst, ce => race, q => raddr(1));	
fd_rad2 	: fdce  port map (c => clkx3p5, d => raddr_d(2), clr => rst, ce => race, q => raddr(2));	
fd_rad3 	: fdce  port map (c => clkx3p5, d => raddr_d(3), clr => rst, ce => race, q => raddr(3));	

raddr_d(0) <= not raddr(0) ;

raddr_d(3 downto 1) <= 	"001" when raddr(3 downto 1) = "000" else
	   	       	"010" when raddr(3 downto 1) = "001" else
	   		"011" when raddr(3 downto 1) = "010" else
	   		"100" when raddr(3 downto 1) = "011" else
	   		"101" when raddr(3 downto 1) = "100" else
	   		"110" when raddr(3 downto 1) = "101" else
			"000" ;

race2_d <= '1' when ramoe = '0' and raddr(3 downto 0) = "1101" else '0' ;
fd_race2 : fdc   port map (c => clkx3p5, d => race2_d,     clr => rst,  q => race2);
	
raddr_d(9 downto 4) <= 	raddr(9 downto 4) + 1 ;

fd_rad4	: fdpe  port map (c => clkx3p5, d => raddr_d(4), pre => rst, ce => race2, q => raddr(4));	
fd_rad5	: fdce  port map (c => clkx3p5, d => raddr_d(5), clr => rst, ce => race2, q => raddr(5));	
fd_rad6	: fdce  port map (c => clkx3p5, d => raddr_d(6), clr => rst, ce => race2, q => raddr(6));	
fd_rad7	: fdce  port map (c => clkx3p5, d => raddr_d(7), clr => rst, ce => race2, q => raddr(7));	
fd_rad8	: fdce  port map (c => clkx3p5, d => raddr_d(8), clr => rst, ce => race2, q => raddr(8));	
fd_rad9	: fdpe  port map (c => clkx3p5, d => raddr_d(9), pre => rst, ce => race2, q => raddr(9));	
	
fd_ramoe :fdc port map(c => clkx3p5, d => notramoe, clr => rst,  q => ramoe) ;	

clkout(1) <= shifterp(0) ;
fd_ca 	: 	fd port map (c => clkx3p5   , d => shifterp(4), q => cint);	
fd_cb 	: 	fd port map (c => clkx3p5not, d => cint,        q => clkout(0));	

shifterp_pre1_d <= '1' when raddr(3 downto 2) = "11" else '0' ;
shifterp_pre2_d <= '1' when raddr(1 downto 0) = "00" and ramoe = '1' else '0' ;
shifterp_pre_d <=  shifterp_pre1 and shifterp_pre2 ;

fd_shpre1 : 	fd  port map (c => clkx3p5, d => shifterp_pre1_d, q => shifterp_pre1);	
fd_shpre2 : 	fd  port map (c => clkx3p5, d => shifterp_pre2_d, q => shifterp_pre2);	
fd_shpre  : 	fd  port map (c => clkx3p5, d => shifterp_pre_d,  q => shifterp_pre);	

shifterp_d <= "1100001" when shifterp_pre = '1' else (shifterp(4 downto 0) & shifterp(6 downto 5)) ;		-- use for 3:4 clock out
--shifterp_d <= "1110001" when shifterp_pre = '1' else (shifterp(4 downto 0) & shifterp(6 downto 5)) ;		-- use for 4:3 clock out

fd_shp0 : fd  port map (c => clkx3p5, d => shifterp_d(0), q => shifterp(0));	
fd_shp1 : fd  port map (c => clkx3p5, d => shifterp_d(1), q => shifterp(1));	
fd_shp2 : fd  port map (c => clkx3p5, d => shifterp_d(2), q => shifterp(2));	
fd_shp3 : fd  port map (c => clkx3p5, d => shifterp_d(3), q => shifterp(3));	
fd_shp4 : fd  port map (c => clkx3p5, d => shifterp_d(4), q => shifterp(4));	
fd_shp5 : fd  port map (c => clkx3p5, d => shifterp_d(5), q => shifterp(5));	
fd_shp6 : fd  port map (c => clkx3p5, d => shifterp_d(6), q => shifterp(6));	
    
process (clk)
begin
if clk'event and clk = '1' then
	dataind <= datain;
end if ;
end process;

ram_raddr <= "00000" & raddr(4 downto 0) ;									-- Use these 2 lines for minimum depth FIFO (32 deep)
ram_waddr <= "00000" & waddr(3 downto 0) ;

--ram_raddr <= raddr ;												-- Use these 2 lines for maximum depth FIFO (1024 deep)
--ram_waddr <= waddr ;

block_ram0 : ramb16_s18_s36 port map(
  	dia	=> "1111111111111111",
  	dib   	=> ramdatain,
  	dipa  	=> "11",      
  	dipb  	=> "1111",
  	ena   	=> ramoe,
  	enb   	=> high,
  	wea   	=> low,
  	web   	=> ramwe,
  	ssra  	=> low,
  	ssrb  	=> low,
  	clka  	=> clkx3p5,
  	clkb  	=> clk,
  	addra 	=> ram_raddr,
  	addrb 	=> ram_waddr,
  	doa   	=> ramdataout,
  	dob   	=> open,
  	dopa  	=> open,
  	dopb  	=> open);
  	
ramdatain <= datain(3 downto 0)  & dataind               when waddr(2 downto 0) = "000" else
             datain(7 downto 0)  & dataind(27 downto 4)  when waddr(2 downto 0) = "001" else
             datain(11 downto 0) & dataind(27 downto 8)  when waddr(2 downto 0) = "010" else
             datain(15 downto 0) & dataind(27 downto 12) when waddr(2 downto 0) = "011" else
             datain(19 downto 0) & dataind(27 downto 16) when waddr(2 downto 0) = "100" else
             datain(23 downto 0) & dataind(27 downto 20) when waddr(2 downto 0) = "101" else
             datain(27 downto 0) & dataind(27 downto 24) ;

dint(0) <= db(8)  when ce1 = '1' else db(0);
dint(1) <= db(9)  when ce1 = '1' else db(1);
dint(2) <= db(10) when ce1 = '1' else db(2);
dint(3) <= db(11) when ce1 = '1' else db(3);
dint(4) <= db(12) when pbp = '1' else db(4);
dint(5) <= db(13) when pbp = '1' else db(5);
dint(6) <= db(14) when pbp = '1' else db(6);
dint(7) <= db(15) when pbp = '1' else db(7);

muxp0 	: fd port map(d => dint(0), c => clkx3p5not, q => dataout(0));
muxp1 	: fd port map(d => dint(1), c => clkx3p5not, q => dataout(1));
muxp2 	: fd port map(d => dint(2), c => clkx3p5not, q => dataout(2));
muxp3 	: fd port map(d => dint(3), c => clkx3p5not, q => dataout(3));
muxp4 	: fd port map(d => dint(4), c => clkx3p5,    q => dataout(4));
muxp5 	: fd port map(d => dint(5), c => clkx3p5,    q => dataout(5));
muxp6 	: fd port map(d => dint(6), c => clkx3p5,    q => dataout(6));
muxp7 	: fd port map(d => dint(7), c => clkx3p5,    q => dataout(7));
	
fd_sp   : fdc port map(c => clkx3p5, d => ramoe, clr => rst, q => syncp);

pbp_d <= syncp;

fd_pbp0 : fdc port map(c => clkx3p5, d => pbp_d,  clr =>rst, q => pbp);

ce0 <= ramoe ;
ce1_d <= not pbp ;

fd_ce1 	: fd  port map (d => ce1_d, c => clkx3p5not, q => ce1);

fd_db0  : fde port map (c => clkx3p5not, d => ramdataout(0),  ce => ce1, q => db(0));
fd_db1  : fde port map (c => clkx3p5not, d => ramdataout(1),  ce => ce1, q => db(1));
fd_db2  : fde port map (c => clkx3p5not, d => ramdataout(2),  ce => ce1, q => db(2));
fd_db3  : fde port map (c => clkx3p5not, d => ramdataout(3),  ce => ce1, q => db(3));
fd_db4  : fde port map (c => clkx3p5,    d => ramdataout(4),  ce => ce0, q => db(4));
fd_db5  : fde port map (c => clkx3p5,    d => ramdataout(5),  ce => ce0, q => db(5));
fd_db6  : fde port map (c => clkx3p5,    d => ramdataout(6),  ce => ce0, q => db(6));
fd_db7  : fde port map (c => clkx3p5,    d => ramdataout(7),  ce => ce0, q => db(7));
fd_db8  : fde port map (c => clkx3p5not, d => ramdataout(8),  ce => ce1, q => db(8));
fd_db9  : fde port map (c => clkx3p5not, d => ramdataout(9),  ce => ce1, q => db(9));
fd_db10 : fde port map (c => clkx3p5not, d => ramdataout(10), ce => ce1, q => db(10));
fd_db11 : fde port map (c => clkx3p5not, d => ramdataout(11), ce => ce1, q => db(11));
fd_db12 : fde port map (c => clkx3p5,    d => ramdataout(12), ce => ce0, q => db(12));
fd_db13 : fde port map (c => clkx3p5,    d => ramdataout(13), ce => ce0, q => db(13));
fd_db14 : fde port map (c => clkx3p5,    d => ramdataout(14), ce => ce0, q => db(14));
fd_db15 : fde port map (c => clkx3p5,    d => ramdataout(15), ce => ce0, q => db(15));

end arch_serdes_4b_7to1_fifo ;
