//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2005                 www.xilinx.com
//
//  XAPP 486 - 7:1 LVDS in Spartan3E Devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :      serdes_5b_7to1_fifo.v
//
//  Description :    generic 5-bit 7:1 serdes with input fifo
// 			data is transmitted LSBs first
// 			0, 5, 10, 15, 20, 25, 30
// 			1, 6, 11, 16, 21, 26, 31
// 			2, 7, 12, 17, 22, 27, 32
// 			3, 8, 13, 18, 23, 28, 33
//			4, 9, 14, 19, 24, 29, 34
//
//			The depth of the FIFO is selectable for either 32 or 1024 deep by
// 			using comments. See the code around line 270.
//
//  Date - revision : November 29th 2006 - v 1.2
//
//  Author :         NJS
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors make and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regardg use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstandg the failure of the
//              essential purpose of any limited remedies herein.
//
//  Copyright  2005 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module serdes_5b_7to1_fifo (
input 		clk,				// clock
input 		clkx3p5,			// 3.5 times clock
input 		clkx3p5not,			// not 3.5 times clock
input 	[34:0]	datain,				// input data
input 		rst,				// reset
output 	[9:0]	dataout,			// output data
output	[1:0]	clkout) ;			// output clock (1x)

wire 		low ;
wire 		high ;
wire 		syncp ;
wire 		ce0   ;
wire 		ce1   ;
wire 	[19:0]	db ;
wire 		pbp   ;
wire 		pbp_d ;
reg 	[34:0]	dataind ;
reg 	[39:0]	ramdatain ;
wire 	[31:0]	ramdataina ;
wire 	[31:0]	ramdatainb ;
wire 	[31:0]	ramdataout ;
wire 	[9:0]	ram_raddr ;
wire 	[8:0]	ram_waddr ;
wire 		ramoe   ;
wire 		notramoe   ;
reg 		ramwe   ;
wire 	[9:0]	raddr ;
reg 	[9:0]	raddr_d ;
reg 	[8:0]	waddr ;
wire 	[6:0]	shifterp ;
wire 	[6:0]	shifterp_d ;
wire 	[9:0]	dint ;
wire 		shifterp_pre  ;
wire 		shifterp_pre1 ;
wire 		shifterp_pre2 ;
wire 		shifterp_pre_d ;
wire 		shifterp_pre1_d ;
wire 		shifterp_pre2_d ;
wire 		race ;
wire 		race_d  ;
wire 		race2 ;
wire 		race2_d  ;
wire 		cint  ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign notramoe = ~ramoe ;

always @ (posedge clk or posedge rst)					// ram write counter, disables write every 8 clocks
begin
if (rst == 1'b1) begin
    	waddr <= 9'b000000000 ;
    	ramwe <= 1'b0;
end
else begin
    	waddr <= waddr + 9'b000000001;
    	if (waddr[2:0] == 3'b110) begin
      		ramwe <= 1'b0;
    	end
    	else begin
      		ramwe <= 1'b1;
    	end
end
end

(* RLOC = "x2y6" *)	FDCE 	fd_rad0	(.C(clkx3p5), .D(raddr_d[0]), .CLR(rst), .CE(ramoe), .Q(raddr[0])) ;	// ram read counter

assign race_d = ~ramoe & raddr[0] ;
(* RLOC = "x2y9" *)	FDC	fd_race	(.C(clkx3p5), .D(race_d),     .CLR(rst),  .Q(race)) ;

(* RLOC = "x2y7" *)	FDCE 	fd_rad1 (.C(clkx3p5), .D(raddr_d[1]), .CLR(rst), .CE(race), .Q(raddr[1])) ;
(* RLOC = "x2y7" *)	FDCE 	fd_rad2 (.C(clkx3p5), .D(raddr_d[2]), .CLR(rst), .CE(race), .Q(raddr[2])) ;
(* RLOC = "x2y8" *)	FDCE 	fd_rad3 (.C(clkx3p5), .D(raddr_d[3]), .CLR(rst), .CE(race), .Q(raddr[3])) ;

always @ (raddr)
begin
raddr_d[0] <= ~raddr[0] ;
case (raddr[3:1])
	3'h0    : raddr_d[3:1] <= 3'h1 ;
	3'h1    : raddr_d[3:1] <= 3'h2 ;
	3'h2    : raddr_d[3:1] <= 3'h3 ;
	3'h3    : raddr_d[3:1] <= 3'h4 ;
	3'h4    : raddr_d[3:1] <= 3'h5 ;
	3'h5    : raddr_d[3:1] <= 3'h6 ;
	default : raddr_d[3:1] <= 3'h0 ;
endcase
raddr_d[9:4] <= raddr[9:4] + 6'b000001 ;
end

assign race2_d = ~ramoe & raddr[3] & raddr[2] & ~raddr[1] & raddr[0] ;
(* RLOC = "x1y7" *)	FDC	fd_race2(.C(clkx3p5), .D(race2_d),    .CLR(rst),  .Q(race2)) ;

(* RLOC = "x2y2" *)	FDPE 	fd_rad4 (.C(clkx3p5), .D(raddr_d[4]), .PRE(rst), .CE(race2), .Q(raddr[4])) ;
(* RLOC = "x2y2" *)	FDCE 	fd_rad5 (.C(clkx3p5), .D(raddr_d[5]), .CLR(rst), .CE(race2), .Q(raddr[5])) ;
(* RLOC = "x2y1" *)	FDCE 	fd_rad6 (.C(clkx3p5), .D(raddr_d[6]), .CLR(rst), .CE(race2), .Q(raddr[6])) ;
(* RLOC = "x2y1" *)	FDCE 	fd_rad7 (.C(clkx3p5), .D(raddr_d[7]), .CLR(rst), .CE(race2), .Q(raddr[7])) ;
(* RLOC = "x3y0" *)	FDCE 	fd_rad8 (.C(clkx3p5), .D(raddr_d[8]), .CLR(rst), .CE(race2), .Q(raddr[8])) ;
(* RLOC = "x3y1" *)	FDPE 	fd_rad9 (.C(clkx3p5), .D(raddr_d[9]), .PRE(rst), .CE(race2), .Q(raddr[9])) ;

(* RLOC = "x1y6" *)	FDC	fd_ramoe	(.C(clkx3p5), .D(notramoe), .CLR(rst), .Q(ramoe)) ;

assign clkout[1] = shifterp[0] ;
(* RLOC = "X2y3", BEL = "FFX" *)	FD	fd_ca	(.C(clkx3p5),    .D(shifterp[4]), .Q(cint)) ;
(* RLOC = "X2y4" *)	FD	fd_cb	(.C(clkx3p5not), .D(cint),        .Q(clkout[0])) ;

assign shifterp_pre1_d = raddr[3] & raddr[2] ;
assign shifterp_pre2_d = ~raddr[1] & ~raddr[0] & ramoe ;
assign shifterp_pre_d =  shifterp_pre1 & shifterp_pre2 ;

(* RLOC = "X1y0" *)	FD	fd_shpre1	(.C(clkx3p5), .D(shifterp_pre1_d), .Q(shifterp_pre1)) ;
(* RLOC = "X1y0" *)	FD	fd_shpre2	(.C(clkx3p5), .D(shifterp_pre2_d), .Q(shifterp_pre2)) ;
(* RLOC = "X1y1" *)	FD	fd_shpre	(.C(clkx3p5), .D(shifterp_pre_d),  .Q(shifterp_pre)) ;

assign shifterp_d = (shifterp_pre == 1'b1) ? 7'b1100001 : {shifterp[4:0], shifterp[6:5]} ;		// use for 3:4 clock out
//assign shifterp_d = (shifterp_pre == 1'b1) ? 7'b1110001 : {shifterp[4:0], shifterp[6:5]} ;		// use for 4:3 clock out

(* RLOC = "x1y2" *)	FD	fd_shp0 	(.C(clkx3p5), .D(shifterp_d[0]), .Q(shifterp[0])) ;
(* RLOC = "x1y2" *)	FD	fd_shp1 	(.C(clkx3p5), .D(shifterp_d[1]), .Q(shifterp[1])) ;
(* RLOC = "x1y3" *)	FD	fd_shp2 	(.C(clkx3p5), .D(shifterp_d[2]), .Q(shifterp[2])) ;
(* RLOC = "x1y3" *)	FD	fd_shp3 	(.C(clkx3p5), .D(shifterp_d[3]), .Q(shifterp[3])) ;
(* RLOC = "x1y4" *)	FD	fd_shp4 	(.C(clkx3p5), .D(shifterp_d[4]), .Q(shifterp[4])) ;
(* RLOC = "x1y4" *)	FD	fd_shp5 	(.C(clkx3p5), .D(shifterp_d[5]), .Q(shifterp[5])) ;
(* RLOC = "x1y5" *)	FD	fd_shp6 	(.C(clkx3p5), .D(shifterp_d[6]), .Q(shifterp[6])) ;

always  @ (posedge clk)
begin
    	dataind <= datain ;
end

//assign ram_raddr = {5'b00000, raddr} ;										// Use these 2 lines for minimum depth FIFO (32 deep)
//assign ram_waddr = {5'b00000, waddr} ;

assign ram_raddr = raddr ;												// Use these 2 lines for maximum depth FIFO (1024 deep)
assign ram_waddr = waddr ;

RAMB16_S18_S36 block_ram0(
  	.DIA	(16'hffff),
  	.DIB   	(ramdataina),
  	.DIPA  	(2'b11),
  	.DIPB  	(4'hf),
  	.ENA   	(ramoe),
  	.ENB   	(high),
  	.WEA   	(low),
  	.WEB   	(ramwe),
  	.SSRA  	(low),
  	.SSRB  	(low),
  	.CLKA  	(clkx3p5),
  	.CLKB  	(clk),
  	.ADDRA 	(ram_raddr),
  	.ADDRB 	(ram_waddr),
  	.DOA   	(ramdataout[15:0]),
  	.DOB   	(),
  	.DOPA  	(),
  	.DOPB  	());

RAMB16_S18_S36 block_ram1(
  	.DIA	(16'hffff),
  	.DIB   	(ramdatainb),
  	.DIPA  	(2'b11),
  	.DIPB  	(4'hf),
  	.ENA   	(ramoe),
  	.ENB   	(high),
  	.WEA   	(low),
  	.WEB   	(ramwe),
  	.SSRA  	(low),
  	.SSRB  	(low),
  	.CLKA  	(clkx3p5),
  	.CLKB  	(clk),
  	.ADDRA 	(ram_raddr),
  	.ADDRB 	(ram_waddr),
  	.DOA   	(ramdataout[31:16]),
  	.DOB   	(),
  	.DOPA  	(),
  	.DOPB  	());

always @ (waddr or datain or dataind)
begin
case (waddr[2:0])
	3'b000  : ramdatain <= {datain[4:0],  dataind[34:0]} ;
	3'b001  : ramdatain <= {datain[9:0],  dataind[34:5]} ;
	3'b010  : ramdatain <= {datain[14:0], dataind[34:10]} ;
	3'b011  : ramdatain <= {datain[19:0], dataind[34:15]} ;
	3'b100  : ramdatain <= {datain[24:0], dataind[34:20]} ;
	3'b101  : ramdatain <= {datain[29:0], dataind[34:25]} ;
	default : ramdatain <= {datain[34:0], dataind[34:30]} ;
endcase
end
assign ramdataina = {6'b111111, ramdatain[29:20], 6'b111111, ramdatain[9:0]} ;
assign ramdatainb = {6'b111111, ramdatain[39:30], 6'b111111, ramdatain[19:10]} ;

assign dint[0] = (db[10] & ce1) | (db[0] & ~ce1) ;
assign dint[1] = (db[11] & ce1) | (db[1] & ~ce1) ;
assign dint[2] = (db[12] & ce1) | (db[2] & ~ce1) ;
assign dint[3] = (db[13] & ce1) | (db[3] & ~ce1) ;
assign dint[4] = (db[14] & ce1) | (db[4] & ~ce1) ;
assign dint[5] = (db[15] & pbp) | (db[5] & ~pbp) ;
assign dint[6] = (db[16] & pbp) | (db[6] & ~pbp) ;
assign dint[7] = (db[17] & pbp) | (db[7] & ~pbp) ;
assign dint[8] = (db[18] & pbp) | (db[8] & ~pbp) ;
assign dint[9] = (db[19] & pbp) | (db[9] & ~pbp) ;

(* RLOC = "x3y2" *)	FD	muxp0  (.D(dint[0]), .C(clkx3p5not), .Q(dataout[0])) ;
(* RLOC = "x3y3" *)	FD	muxp1  (.D(dint[1]), .C(clkx3p5not), .Q(dataout[1])) ;
(* RLOC = "x3y3" *)	FD	muxp2  (.D(dint[2]), .C(clkx3p5not), .Q(dataout[2])) ;
(* RLOC = "x3y4" *)	FD	muxp3  (.D(dint[3]), .C(clkx3p5not), .Q(dataout[3])) ;
(* RLOC = "x3y4" *)	FD	muxp4  (.D(dint[4]), .C(clkx3p5not), .Q(dataout[4])) ;
(* RLOC = "x3y5" *)	FD	muxp5  (.D(dint[5]), .C(clkx3p5),    .Q(dataout[5])) ;
(* RLOC = "x3y5" *)	FD	muxp6  (.D(dint[6]), .C(clkx3p5),    .Q(dataout[6])) ;
(* RLOC = "x3y6" *)	FD	muxp7  (.D(dint[7]), .C(clkx3p5),    .Q(dataout[7])) ;
(* RLOC = "x3y6" *)	FD	muxp8  (.D(dint[8]), .C(clkx3p5),    .Q(dataout[8])) ;
(* RLOC = "x3y7" *)	FD	muxp9  (.D(dint[9]), .C(clkx3p5),    .Q(dataout[9])) ;

(* RLOC = "x3y9" *)	FDC	fd_sp	(.C(clkx3p5), .D(ramoe), .CLR(rst), .Q(syncp)) ;

assign pbp_d = syncp;

(* RLOC = "x3y9" *)	FDC 	fd_pbp0	(.C(clkx3p5), .D(pbp_d),  .CLR(rst), .Q(pbp)) ;

assign ce0 = ramoe ;
assign ce1_d = ~pbp ;
(* RLOC = "x3y8", BEL = "FFX" *)	FD	fd_ce1	(.C(clkx3p5not), .D(ce1_d), .Q(ce1)) ;

(* RLOC = "x0y0" *)	FDE	fd_db0  (.C(clkx3p5not), .D(ramdataout[0]),  .CE(ce1), .Q(db[0])) ;
(* RLOC = "x0y0" *)	FDE	fd_db1  (.C(clkx3p5not), .D(ramdataout[1]),  .CE(ce1), .Q(db[1])) ;
(* RLOC = "x0y1" *)	FDE	fd_db2  (.C(clkx3p5not), .D(ramdataout[2]),  .CE(ce1), .Q(db[2])) ;
(* RLOC = "x0y1" *)	FDE	fd_db3  (.C(clkx3p5not), .D(ramdataout[3]),  .CE(ce1), .Q(db[3])) ;
(* RLOC = "x0y2" *)	FDE	fd_db4  (.C(clkx3p5not), .D(ramdataout[4]),  .CE(ce1), .Q(db[4])) ;
(* RLOC = "x0y3" *)	FDE	fd_db5  (.C(clkx3p5),    .D(ramdataout[5]),  .CE(ce0), .Q(db[5])) ;
(* RLOC = "x0y3" *)	FDE	fd_db6  (.C(clkx3p5),    .D(ramdataout[6]),  .CE(ce0), .Q(db[6])) ;
(* RLOC = "x0y4" *)	FDE	fd_db7  (.C(clkx3p5),    .D(ramdataout[7]),  .CE(ce0), .Q(db[7])) ;
(* RLOC = "x0y4" *)	FDE	fd_db8  (.C(clkx3p5),    .D(ramdataout[8]),  .CE(ce0), .Q(db[8])) ;
(* RLOC = "x0y5" *)	FDE	fd_db9  (.C(clkx3p5),    .D(ramdataout[9]),  .CE(ce0), .Q(db[9])) ;
(* RLOC = "x0y8" *)	FDE	fd_db10 (.C(clkx3p5not), .D(ramdataout[16]), .CE(ce1), .Q(db[10])) ;
(* RLOC = "x0y8" *)	FDE	fd_db11 (.C(clkx3p5not), .D(ramdataout[17]), .CE(ce1), .Q(db[11])) ;
(* RLOC = "x0y9" *)	FDE	fd_db12 (.C(clkx3p5not), .D(ramdataout[18]), .CE(ce1), .Q(db[12])) ;
(* RLOC = "x0y9" *)	FDE	fd_db13 (.C(clkx3p5not), .D(ramdataout[19]), .CE(ce1), .Q(db[13])) ;
(* RLOC = "x1y9" *)	FDE	fd_db14 (.C(clkx3p5not), .D(ramdataout[20]), .CE(ce1), .Q(db[14])) ;
(* RLOC = "x0y5" *)	FDE	fd_db15 (.C(clkx3p5),    .D(ramdataout[21]), .CE(ce0), .Q(db[15])) ;
(* RLOC = "x0y6" *)	FDE	fd_db16 (.C(clkx3p5),    .D(ramdataout[22]), .CE(ce0), .Q(db[16])) ;
(* RLOC = "x0y6" *)	FDE	fd_db17 (.C(clkx3p5),    .D(ramdataout[23]), .CE(ce0), .Q(db[17])) ;
(* RLOC = "x0y7" *)	FDE	fd_db18 (.C(clkx3p5),    .D(ramdataout[24]), .CE(ce0), .Q(db[18])) ;
(* RLOC = "x0y7" *)	FDE	fd_db19 (.C(clkx3p5),    .D(ramdataout[25]), .CE(ce0), .Q(db[19])) ;

endmodule
