------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2006                 www.xilinx.com
--
--  XAPP 486 - 7:1 LVDS in Spartan3E Devices
--
------------------------------------------------------------------------------
--
--  File name :       serdes_5b_7to1.vhd
--
--  Description :     generic 5-bit 7:1 serdes for Spartan 3E, now using ODDR2 with ALIGNMENT = NONE
-- 			data is transmitted LSBs first
-- 			0, 5, 10, 15, 20, 25, 30
-- 			1, 6, 11, 16, 21, 26, 31
-- 			2, 7, 12, 17, 22, 27, 32
-- 			3, 8, 13, 18, 23, 28, 33
--			4, 9, 14, 19, 24, 29, 34
--
--  Date - revision : October 16th 2006 - v 1.4
--			
--			Version 1.4 : 	Brings the DDR registers to the top level and no
--					longer uses 'C0' alignment
--
--  Author :          NJS
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2006 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------	
-- 
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

library unisim ;
use unisim.vcomponents.all ;

entity serdes_5b_7to1 is port (					
	clk 		: in std_logic ;			-- clock
	clkx3p5 	: in std_logic ;			-- 3.5 times clock
	clkx3p5not	: in std_logic ;			-- not 3.5 times clock
	datain 		: in std_logic_vector(34 downto 0) ;	-- input data
	rst 		: in std_logic ;			-- reset
	dataout		: out std_logic_vector(9 downto 0) ;	-- output data
	clkout		: out std_logic_vector(1 downto 0)) ;	-- output clock (1x)
end serdes_5b_7to1;

architecture arch_serdes_5b_7to1 of serdes_5b_7to1 is

signal low 		: std_logic ;
signal high 		: std_logic ;
signal clkd2 		: std_logic ;
signal clkd2d_a 	: std_logic ;
signal clkd2d_b 	: std_logic ;
signal xra 		: std_logic ;
signal xrareg 		: std_logic ;
signal notclk3p5d2 	: std_logic ;
signal clk3p5d2a 	: std_logic ;
signal clk3p5d2b	: std_logic ;
signal ce0 		: std_logic ;
signal ce1 		: std_logic ;
signal ce0_d 		: std_logic ;
signal ce1_d 		: std_logic ;
signal phase_bna 	: std_logic_vector(2 downto 0) ;
signal phase_bnb 	: std_logic_vector(2 downto 0) ;
signal pba 		: std_logic_vector(2 downto 0) ;
signal pbb		: std_logic_vector(2 downto 0) ;
signal pbad 		: std_logic_vector(2 downto 0) ;
signal pbadd 		: std_logic_vector(2 downto 0) ;
signal mux_ap 		: std_logic_vector(5 downto 0) ;
signal mux_ap_d		: std_logic_vector(6 downto 0) ;
signal mux_bp 		: std_logic_vector(5 downto 0) ;
signal mux_bp_d		: std_logic_vector(6 downto 0) ;
signal mux_cp 		: std_logic_vector(5 downto 0) ;
signal mux_cp_d		: std_logic_vector(6 downto 0) ;
signal mux_dp 		: std_logic_vector(5 downto 0) ;
signal mux_dp_d		: std_logic_vector(6 downto 0) ;
signal mux_ep 		: std_logic_vector(5 downto 0) ;
signal mux_ep_d		: std_logic_vector(6 downto 0) ;
signal mux_an 		: std_logic_vector(5 downto 0) ;
signal mux_an_d		: std_logic_vector(6 downto 0) ;
signal mux_bn 		: std_logic_vector(5 downto 0) ;
signal mux_bn_d		: std_logic_vector(6 downto 0) ;
signal mux_cn 		: std_logic_vector(5 downto 0) ;
signal mux_cn_d		: std_logic_vector(6 downto 0) ;
signal mux_dn 		: std_logic_vector(5 downto 0) ;
signal mux_dn_d		: std_logic_vector(6 downto 0) ;
signal mux_en 		: std_logic_vector(5 downto 0) ;
signal mux_en_d		: std_logic_vector(6 downto 0) ;
signal mux_p 		: std_logic_vector(4 downto 0) ;
signal mux_n 		: std_logic_vector(4 downto 0) ;
signal db 		: std_logic_vector(34 downto 0) ;
signal dataint	 	: std_logic_vector(34 downto 0) ;
signal datainr 		: std_logic_vector(34 downto 25) ;
signal clkcp 		: std_logic ;
signal clkcn 		: std_logic ;
signal clkcp_d 		: std_logic ;
signal clkcn_d 		: std_logic ;

attribute init		: string ;
attribute bel		: string ;
attribute rloc		: string ;
attribute syn_replicate : boolean ;					-- stops synplicity duplicating regs

attribute syn_replicate of fdcpba0 : label is false ;
attribute syn_replicate of fdcpba1 : label is false ;
attribute syn_replicate of fdcpba2 : label is false ;
attribute syn_replicate of fdcpba3 : label is false ;
attribute syn_replicate of fdcpba4 : label is false ;
attribute syn_replicate of fdcpba5 : label is false ;
attribute syn_replicate of fdcpbb0 : label is false ;
attribute syn_replicate of fdcpbb1 : label is false ;
attribute syn_replicate of fdcpbb2 : label is false ;

constant xa : string := "12" ;						-- Use this set of 4 constants for macro either side of block RAM
constant xb : string := "13" ;
constant xc : string := "14" ;
constant xd : string := "15" ;

--constant xa : string := "4" ;						-- Use this set of 4 constants for contiguous macro 
--constant xb : string := "5" ;						-- Used at the top of the device or right side of 100E
--constant xc : string := "6" ;
--constant xd : string := "7" ;

attribute rloc of fd_c35d2a	: label is "x" & xb & "y1" ;
attribute rloc of fd_c35d2b	: label is "x" & xb & "y1" ;
attribute bel  of fd_cb		: label is "FFX" ;
attribute rloc of fd_cb		: label is "x" & xb & "y2" ;
attribute bel  of fd_cbda	: label is "FFX" ;
attribute rloc of fd_cbda	: label is "x" & xb & "y3" ;
attribute rloc of fd_cbdb	: label is "x" & xb & "y3" ;

attribute rloc of fd_xra	: label is "x" & xb & "y6" ;

attribute rloc of fdcpba0	: label is "x" & xa & "y4" ;
attribute rloc of fdcpba1	: label is "x" & xa & "y4" ;
attribute rloc of fdcpba2	: label is "x" & xb & "y7" ;
attribute rloc of fdcpba3	: label is "x" & xc & "y4" ;
attribute rloc of fdcpba4	: label is "x" & xb & "y7" ;
attribute rloc of fdcpba5	: label is "x" & xc & "y4" ;
attribute rloc of fd_ce0	: label is "x2y5" ;
attribute rloc of fd_ce1	: label is "x2y5" ;

attribute rloc of fdcpbb0	: label is "x2y4" ;
attribute rloc of fdcpbb1	: label is "x2y4" ;
attribute rloc of fdcpbb2	: label is "x2y3" ;

attribute rloc of fd_db0	: label is "x1y0" ;
attribute rloc of fd_db1	: label is "x1y2" ;
attribute rloc of fd_db2	: label is "x1y4" ;
attribute rloc of fd_db3	: label is "x1y6" ;
attribute rloc of fd_db4	: label is "x1y8" ;

attribute rloc of fd_db5	: label is "x0y0" ;
attribute rloc of fd_db6	: label is "x0y2" ;
attribute rloc of fd_db7	: label is "x0y4" ;
attribute rloc of fd_db8	: label is "x0y6" ;
attribute rloc of fd_db9	: label is "x0y8" ;

attribute rloc of fd_db10	: label is "x1y0" ;
attribute rloc of fd_db11	: label is "x1y2" ;
attribute rloc of fd_db12	: label is "x1y4" ;
attribute rloc of fd_db13	: label is "x1y6" ;
attribute rloc of fd_db14	: label is "x1y8" ;

attribute rloc of fd_db15	: label is "x0y0" ;
attribute rloc of fd_db16	: label is "x0y2" ;
attribute rloc of fd_db17	: label is "x0y4" ;
attribute rloc of fd_db18	: label is "x0y6" ;
attribute rloc of fd_db19	: label is "x0y8" ;

attribute rloc of fd_db20	: label is "x1y1" ;
attribute rloc of fd_db21	: label is "x1y3" ;
attribute rloc of fd_db22	: label is "x1y5" ;
attribute rloc of fd_db23	: label is "x1y7" ;
attribute rloc of fd_db24	: label is "x1y9" ;

attribute rloc of fd_db25	: label is "x0y1" ;
attribute rloc of fd_db26	: label is "x0y3" ;
attribute rloc of fd_db27	: label is "x0y5" ;
attribute rloc of fd_db28	: label is "x0y7" ;
attribute rloc of fd_db29	: label is "x0y9" ;

attribute rloc of fd_db30	: label is "x0y1" ;
attribute rloc of fd_db31	: label is "x0y3" ;
attribute rloc of fd_db32	: label is "x0y5" ;
attribute rloc of fd_db33	: label is "x0y7" ;
attribute rloc of fd_db34	: label is "x0y9" ;

attribute rloc of fd_d25	: label is "x2y1" ;
attribute rloc of fd_d26	: label is "x2y1" ;
attribute rloc of fd_d27	: label is "x2y7" ;
attribute rloc of fd_d28	: label is "x2y7" ;
attribute rloc of fd_d29	: label is "x2y9" ;

attribute rloc of fd_d30	: label is "x2y0" ;
attribute rloc of fd_d31	: label is "x2y0" ;
attribute rloc of fd_d32	: label is "x2y6" ;
attribute rloc of fd_d33	: label is "x2y6" ;
attribute rloc of fd_d34	: label is "x2y9" ;

attribute rloc of muxap0	: label is "x" & xa & "y1" ;
attribute rloc of muxap1	: label is "x" & xa & "y1" ;
attribute rloc of muxap2	: label is "x3y1" ;
attribute rloc of muxap3	: label is "x3y1" ;
attribute rloc of muxap4	: label is "x" & xd & "y0" ;
attribute rloc of muxap5	: label is "x" & xd & "y0" ;
attribute rloc of muxap6	: label is "x" & xc & "y2" ;

attribute rloc of muxbp0	: label is "x" & xa & "y3" ;
attribute rloc of muxbp1	: label is "x" & xa & "y3" ;
attribute rloc of muxbp2	: label is "x3y3" ;
attribute rloc of muxbp3	: label is "x3y3" ;
attribute rloc of muxbp4	: label is "x" & xd & "y2" ;
attribute rloc of muxbp5	: label is "x" & xd & "y2" ;
attribute rloc of muxbp6	: label is "x" & xc & "y0" ;

attribute rloc of muxcp0	: label is "x" & xa & "y5" ;
attribute rloc of muxcp1	: label is "x" & xa & "y5" ;
attribute rloc of muxcp2	: label is "x3y5" ;
attribute rloc of muxcp3	: label is "x3y5" ;
attribute rloc of muxcp4	: label is "x" & xd & "y4" ;
attribute rloc of muxcp5	: label is "x" & xd & "y4" ;
attribute rloc of muxcp6	: label is "x" & xb & "y4" ;

attribute rloc of muxdp0	: label is "x" & xa & "y7" ;
attribute rloc of muxdp1	: label is "x" & xa & "y7" ;
attribute rloc of muxdp2	: label is "x3y7" ;
attribute rloc of muxdp3	: label is "x3y7" ;
attribute rloc of muxdp4	: label is "x" & xd & "y6" ;
attribute rloc of muxdp5	: label is "x" & xd & "y6" ;
attribute rloc of muxdp6	: label is "x" & xc & "y6" ;

attribute rloc of muxep0	: label is "x" & xa & "y9" ;
attribute rloc of muxep1	: label is "x" & xa & "y9" ;
attribute rloc of muxep2	: label is "x3y9" ;
attribute rloc of muxep3	: label is "x3y9" ;
attribute rloc of muxep4	: label is "x" & xd & "y8" ;
attribute rloc of muxep5	: label is "x" & xd & "y8" ;
attribute rloc of muxep6	: label is "x" & xc & "y8" ;

attribute rloc of muxan0	: label is "x" & xa & "y0" ;
attribute rloc of muxan1	: label is "x" & xa & "y0" ;
attribute rloc of muxan2	: label is "x3y0" ;
attribute rloc of muxan3	: label is "x3y0" ;
attribute rloc of muxan4	: label is "x" & xd & "y1" ;
attribute rloc of muxan5	: label is "x" & xd & "y1" ;
attribute rloc of muxan6	: label is "x" & xb & "y0" ;

attribute rloc of muxbn0	: label is "x2y2" ;
attribute rloc of muxbn1	: label is "x2y2" ;
attribute rloc of muxbn2	: label is "x3y2" ;
attribute rloc of muxbn3	: label is "x3y2" ;
attribute rloc of muxbn4	: label is "x" & xd & "y3" ;
attribute rloc of muxbn5	: label is "x" & xd & "y3" ;
attribute rloc of muxbn6	: label is "x" & xb & "y0" ;

attribute rloc of muxcn0	: label is "x" & xa & "y2" ;
attribute rloc of muxcn1	: label is "x" & xa & "y2" ;
attribute rloc of muxcn2	: label is "x3y4" ;
attribute rloc of muxcn3	: label is "x3y4" ;
attribute rloc of muxcn4	: label is "x" & xd & "y5" ;
attribute rloc of muxcn5	: label is "x" & xd & "y5" ;
attribute rloc of muxcn6	: label is "x" & xc & "y5" ;

attribute rloc of muxdn0	: label is "x" & xa & "y6" ;
attribute rloc of muxdn1	: label is "x" & xa & "y6" ;
attribute rloc of muxdn2	: label is "x3y6" ;
attribute rloc of muxdn3	: label is "x3y6" ;
attribute rloc of muxdn4	: label is "x" & xd & "y7" ;
attribute rloc of muxdn5	: label is "x" & xd & "y7" ;
attribute rloc of muxdn6	: label is "x" & xc & "y5" ;

attribute rloc of muxen0	: label is "x" & xa & "y8" ;
attribute rloc of muxen1	: label is "x" & xa & "y8" ;
attribute rloc of muxen2	: label is "x3y8" ;
attribute rloc of muxen3	: label is "x3y8" ;
attribute rloc of muxen4	: label is "x" & xd & "y9" ;
attribute rloc of muxen5	: label is "x" & xd & "y9" ;
attribute rloc of muxen6	: label is "x" & xc & "y6" ;

attribute rloc of fd_clkcp	: label is "x" & xb & "y8" ;
attribute rloc of fd_clkcn	: label is "x" & xb & "y8" ;

attribute rloc of fd_mnn0	: label is "x" & xc & "y3" ;
attribute rloc of fd_mnn1	: label is "x" & xc & "y1" ;
attribute rloc of fd_mnn2	: label is "x" & xb & "y5" ;
attribute rloc of fd_mnn3	: label is "x" & xc & "y7" ;
attribute rloc of fd_mnn4	: label is "x" & xc & "y9" ;
attribute rloc of fd_mnn5	: label is "x" & xb & "y9" ;

attribute bel  of fd_mnn0	: label is "FFX" ;
attribute bel  of fd_mnn1	: label is "FFX" ;
attribute bel  of fd_mnn2	: label is "FFX" ;
attribute bel  of fd_mnn3	: label is "FFX" ;
attribute bel  of fd_mnn4	: label is "FFX" ;
attribute bel  of fd_mnn5	: label is "FFX" ;

begin

low <= '0' ;
high <= '1' ;
dataint <= datain after 100 pS ; 						-- this is to get around an issue that appeared 
										-- in simulation with MTI 6.0
 	
mux_an_d(0)  <= db(0)  when pba(0) = '0' else db(10) ;
mux_an_d(1)  <= db(20) when pba(0) = '0' else db(30) ;
mux_an_d(2)  <= db(5)  when pbb(0) = '0' else db(15) ;
mux_an_d(3)  <= db(25) ;
mux_an_d(4)  <= mux_an(0)  when pbad(1)  = '0' else mux_an(1) ;
mux_an_d(5)  <= mux_an(2)  when pbad(1)  = '0' else mux_an(3) ;
mux_an_d(6)  <= mux_an(4)  when pbadd(2) = '0' else mux_an(5) ;

muxan0 	: fd  	port map(d => mux_an_d(0),  c => clkx3p5, q => mux_an(0)) ;
muxan1 	: fd  	port map(d => mux_an_d(1),  c => clkx3p5, q => mux_an(1)) ;
muxan2 	: fd  	port map(d => mux_an_d(2),  c => clkx3p5, q => mux_an(2)) ;
muxan3 	: fd  	port map(d => mux_an_d(3),  c => clkx3p5, q => mux_an(3)) ;
muxan4 	: fd  	port map(d => mux_an_d(4),  c => clkx3p5, q => mux_an(4)) ;
muxan5 	: fd  	port map(d => mux_an_d(5),  c => clkx3p5, q => mux_an(5)) ;
muxan6 	: fd  	port map(d => mux_an_d(6),  c => clkx3p5, q => dataout(5)) ;

mux_ap_d(0)  <= db(5)  when pba(0) = '0' else db(15) ;
mux_ap_d(1)  <= db(25) when pba(0) = '0' else db(0) ;
mux_ap_d(2)  <= db(10) when pbb(0) = '0' else db(20) ;
mux_ap_d(3)  <= db(30) ;
mux_ap_d(4)  <= mux_ap(0)  when pbad(1)  = '0' else mux_ap(1) ;
mux_ap_d(5)  <= mux_ap(2)  when pbad(1)  = '0' else mux_ap(3) ;
mux_ap_d(6)  <= mux_ap(4)  when pbadd(2) = '0' else mux_ap(5) ;

muxap0 	: fd  	port map(d => mux_ap_d(0),  c => clkx3p5, q => mux_ap(0)) ;
muxap1 	: fd  	port map(d => mux_ap_d(1),  c => clkx3p5, q => mux_ap(1)) ;
muxap2 	: fd  	port map(d => mux_ap_d(2),  c => clkx3p5, q => mux_ap(2)) ;
muxap3 	: fd  	port map(d => mux_ap_d(3),  c => clkx3p5, q => mux_ap(3)) ;
muxap4 	: fd  	port map(d => mux_ap_d(4),  c => clkx3p5, q => mux_ap(4)) ;
muxap5 	: fd  	port map(d => mux_ap_d(5),  c => clkx3p5, q => mux_ap(5)) ;
muxap6 	: fd  	port map(d => mux_ap_d(6),  c => clkx3p5, q => mux_p(0)) ;

mux_bn_d(0)  <= db(1)  when pbb(0) = '0' else db(11) ;
mux_bn_d(1)  <= db(21) when pbb(0) = '0' else db(31) ;
mux_bn_d(2)  <= db(6)  when pbb(0) = '0' else db(16) ;
mux_bn_d(3)  <= db(26) ;
mux_bn_d(4)  <= mux_bn(0)  when pbad(1)  = '0' else mux_bn(1) ;
mux_bn_d(5)  <= mux_bn(2)  when pbad(1)  = '0' else mux_bn(3) ;
mux_bn_d(6)  <= mux_bn(4)  when pbadd(2) = '0' else mux_bn(5) ;

muxbn0 	: fd  	port map(d => mux_bn_d(0),  c => clkx3p5, q => mux_bn(0)) ;
muxbn1 	: fd  	port map(d => mux_bn_d(1),  c => clkx3p5, q => mux_bn(1)) ;
muxbn2 	: fd  	port map(d => mux_bn_d(2),  c => clkx3p5, q => mux_bn(2)) ;
muxbn3 	: fd  	port map(d => mux_bn_d(3),  c => clkx3p5, q => mux_bn(3)) ;
muxbn4 	: fd  	port map(d => mux_bn_d(4),  c => clkx3p5, q => mux_bn(4)) ;
muxbn5 	: fd  	port map(d => mux_bn_d(5),  c => clkx3p5, q => mux_bn(5)) ;
muxbn6 	: fd  	port map(d => mux_bn_d(6),  c => clkx3p5, q => dataout(6)) ;

mux_bp_d(0)  <= db(6)  when pba(0) = '0' else db(16) ;
mux_bp_d(1)  <= db(26) when pba(0) = '0' else db(1) ;
mux_bp_d(2)  <= db(11) when pbb(0) = '0' else db(21) ;
mux_bp_d(3)  <= db(31) ;
mux_bp_d(4)  <= mux_bp(0)  when pbad(1)  = '0' else mux_bp(1) ;
mux_bp_d(5)  <= mux_bp(2)  when pbad(1)  = '0' else mux_bp(3) ;
mux_bp_d(6)  <= mux_bp(4)  when pbadd(2) = '0' else mux_bp(5) ;

muxbp0 	: fd  	port map(d => mux_bp_d(0),  c => clkx3p5, q => mux_bp(0)) ;
muxbp1 	: fd  	port map(d => mux_bp_d(1),  c => clkx3p5, q => mux_bp(1)) ;
muxbp2 	: fd  	port map(d => mux_bp_d(2),  c => clkx3p5, q => mux_bp(2)) ;
muxbp3 	: fd  	port map(d => mux_bp_d(3),  c => clkx3p5, q => mux_bp(3)) ;
muxbp4 	: fd  	port map(d => mux_bp_d(4),  c => clkx3p5, q => mux_bp(4)) ;
muxbp5 	: fd  	port map(d => mux_bp_d(5),  c => clkx3p5, q => mux_bp(5)) ;
muxbp6 	: fd  	port map(d => mux_bp_d(6),  c => clkx3p5, q => mux_p(1)) ;

mux_cn_d(0)  <= db(2)  when pba(0) = '0' else db(12) ;
mux_cn_d(1)  <= db(22) when pba(0) = '0' else db(32) ;
mux_cn_d(2)  <= db(7)  when pbb(0) = '0' else db(17) ;
mux_cn_d(3)  <= db(27) ;
mux_cn_d(4)  <= mux_cn(0)  when pbad(1)  = '0' else mux_cn(1) ;
mux_cn_d(5)  <= mux_cn(2)  when pbad(1)  = '0' else mux_cn(3) ;
mux_cn_d(6)  <= mux_cn(4)  when pbadd(2) = '0' else mux_cn(5) ;

muxcn0 	: fd  	port map(d => mux_cn_d(0),  c => clkx3p5, q => mux_cn(0)) ;
muxcn1 	: fd  	port map(d => mux_cn_d(1),  c => clkx3p5, q => mux_cn(1)) ;
muxcn2 	: fd  	port map(d => mux_cn_d(2),  c => clkx3p5, q => mux_cn(2)) ;
muxcn3 	: fd  	port map(d => mux_cn_d(3),  c => clkx3p5, q => mux_cn(3)) ;
muxcn4 	: fd  	port map(d => mux_cn_d(4),  c => clkx3p5, q => mux_cn(4)) ;
muxcn5 	: fd  	port map(d => mux_cn_d(5),  c => clkx3p5, q => mux_cn(5)) ;
muxcn6 	: fd  	port map(d => mux_cn_d(6),  c => clkx3p5, q => dataout(7)) ;

mux_cp_d(0)  <= db(7)  when pba(0) = '0' else db(17) ;
mux_cp_d(1)  <= db(27) when pba(0) = '0' else db(2) ;
mux_cp_d(2)  <= db(12) when pbb(0) = '0' else db(22) ;
mux_cp_d(3)  <= db(32) ;
mux_cp_d(4)  <= mux_cp(0)  when pbad(1)  = '0' else mux_cp(1) ;
mux_cp_d(5)  <= mux_cp(2)  when pbad(1)  = '0' else mux_cp(3) ;
mux_cp_d(6)  <= mux_cp(4)  when pbadd(2) = '0' else mux_cp(5) ;

muxcp0 	: fd  	port map(d => mux_cp_d(0),  c => clkx3p5, q => mux_cp(0)) ;
muxcp1 	: fd  	port map(d => mux_cp_d(1),  c => clkx3p5, q => mux_cp(1)) ;
muxcp2 	: fd  	port map(d => mux_cp_d(2),  c => clkx3p5, q => mux_cp(2)) ;
muxcp3 	: fd  	port map(d => mux_cp_d(3),  c => clkx3p5, q => mux_cp(3)) ;
muxcp4 	: fd  	port map(d => mux_cp_d(4),  c => clkx3p5, q => mux_cp(4)) ;
muxcp5 	: fd  	port map(d => mux_cp_d(5),  c => clkx3p5, q => mux_cp(5)) ;
muxcp6 	: fd  	port map(d => mux_cp_d(6),  c => clkx3p5, q => mux_p(2)) ;

mux_dn_d(0)  <= db(3)  when pba(0) = '0' else db(13) ;
mux_dn_d(1)  <= db(23) when pba(0) = '0' else db(33) ;
mux_dn_d(2)  <= db(8)  when pbb(0) = '0' else db(18) ;
mux_dn_d(3)  <= db(28) ;
mux_dn_d(4)  <= mux_dn(0)  when pbad(1)  = '0' else mux_dn(1) ;
mux_dn_d(5)  <= mux_dn(2)  when pbad(1)  = '0' else mux_dn(3) ;
mux_dn_d(6)  <= mux_dn(4)  when pbadd(2) = '0' else mux_dn(5) ;

muxdn0 	: fd  	port map(d => mux_dn_d(0),  c => clkx3p5, q => mux_dn(0)) ;
muxdn1 	: fd  	port map(d => mux_dn_d(1),  c => clkx3p5, q => mux_dn(1)) ;
muxdn2 	: fd  	port map(d => mux_dn_d(2),  c => clkx3p5, q => mux_dn(2)) ;
muxdn3 	: fd  	port map(d => mux_dn_d(3),  c => clkx3p5, q => mux_dn(3)) ;
muxdn4 	: fd  	port map(d => mux_dn_d(4),  c => clkx3p5, q => mux_dn(4)) ;
muxdn5 	: fd  	port map(d => mux_dn_d(5),  c => clkx3p5, q => mux_dn(5)) ;
muxdn6 	: fd  	port map(d => mux_dn_d(6),  c => clkx3p5, q => dataout(8)) ;

mux_dp_d(0)  <= db(8)  when pba(0) = '0' else db(18) ;
mux_dp_d(1)  <= db(28) when pba(0) = '0' else db(3) ;
mux_dp_d(2)  <= db(13) when pbb(0) = '0' else db(23) ;
mux_dp_d(3)  <= db(33) ;
mux_dp_d(4)  <= mux_dp(0)  when pbad(1)  = '0' else mux_dp(1) ;
mux_dp_d(5)  <= mux_dp(2)  when pbad(1)  = '0' else mux_dp(3) ;
mux_dp_d(6)  <= mux_dp(4)  when pbadd(2) = '0' else mux_dp(5) ;

muxdp0 	: fd  	port map(d => mux_dp_d(0),  c => clkx3p5, q => mux_dp(0)) ;
muxdp1 	: fd  	port map(d => mux_dp_d(1),  c => clkx3p5, q => mux_dp(1)) ;
muxdp2 	: fd  	port map(d => mux_dp_d(2),  c => clkx3p5, q => mux_dp(2)) ;
muxdp3 	: fd  	port map(d => mux_dp_d(3),  c => clkx3p5, q => mux_dp(3)) ;
muxdp4 	: fd  	port map(d => mux_dp_d(4),  c => clkx3p5, q => mux_dp(4)) ;
muxdp5 	: fd  	port map(d => mux_dp_d(5),  c => clkx3p5, q => mux_dp(5)) ;
muxdp6 	: fd  	port map(d => mux_dp_d(6),  c => clkx3p5, q => mux_p(3)) ;

mux_en_d(0)  <= db(4)  when pba(0) = '0' else db(14) ;
mux_en_d(1)  <= db(24) when pba(0) = '0' else db(34) ;
mux_en_d(2)  <= db(9)  when pbb(0) = '0' else db(19) ;
mux_en_d(3)  <= db(29) ;
mux_en_d(4)  <= mux_en(0)  when pbad(1)  = '0' else mux_en(1) ;
mux_en_d(5)  <= mux_en(2)  when pbad(1)  = '0' else mux_en(3) ;
mux_en_d(6)  <= mux_en(4)  when pbadd(2) = '0' else mux_en(5) ;

muxen0 	: fd  	port map(d => mux_en_d(0),  c => clkx3p5, q => mux_en(0)) ;
muxen1 	: fd  	port map(d => mux_en_d(1),  c => clkx3p5, q => mux_en(1)) ;
muxen2 	: fd  	port map(d => mux_en_d(2),  c => clkx3p5, q => mux_en(2)) ;
muxen3 	: fd  	port map(d => mux_en_d(3),  c => clkx3p5, q => mux_en(3)) ;
muxen4 	: fd  	port map(d => mux_en_d(4),  c => clkx3p5, q => mux_en(4)) ;
muxen5 	: fd  	port map(d => mux_en_d(5),  c => clkx3p5, q => mux_en(5)) ;
muxen6 	: fd  	port map(d => mux_en_d(6),  c => clkx3p5, q => dataout(9)) ;

mux_ep_d(0)  <= db(9)  when pba(0) = '0' else db(19) ;
mux_ep_d(1)  <= db(29) when pba(0) = '0' else db(4) ;
mux_ep_d(2)  <= db(14) when pbb(0) = '0' else db(24) ;
mux_ep_d(3)  <= db(34) ;
mux_ep_d(4)  <= mux_ep(0)  when pbad(1)  = '0' else mux_ep(1) ;
mux_ep_d(5)  <= mux_ep(2)  when pbad(1)  = '0' else mux_ep(3) ;
mux_ep_d(6)  <= mux_ep(4)  when pbadd(2) = '0' else mux_ep(5) ;

muxep0 	: fd  	port map(d => mux_ep_d(0),  c => clkx3p5, q => mux_ep(0)) ;
muxep1 	: fd  	port map(d => mux_ep_d(1),  c => clkx3p5, q => mux_ep(1)) ;
muxep2 	: fd  	port map(d => mux_ep_d(2),  c => clkx3p5, q => mux_ep(2)) ;
muxep3 	: fd  	port map(d => mux_ep_d(3),  c => clkx3p5, q => mux_ep(3)) ;
muxep4 	: fd  	port map(d => mux_ep_d(4),  c => clkx3p5, q => mux_ep(4)) ;
muxep5 	: fd  	port map(d => mux_ep_d(5),  c => clkx3p5, q => mux_ep(5)) ;
muxep6 	: fd  	port map(d => mux_ep_d(6),  c => clkx3p5, q => mux_p(4)) ;

fd_mnn0	: fd  	port map(d => mux_p(0),  c => clkx3p5not, q => dataout(0)) ;
fd_mnn1	: fd  	port map(d => mux_p(1),  c => clkx3p5not, q => dataout(1)) ;
fd_mnn2	: fd  	port map(d => mux_p(2),  c => clkx3p5not, q => dataout(2)) ;
fd_mnn3	: fd  	port map(d => mux_p(3),  c => clkx3p5not, q => dataout(3)) ;
fd_mnn4	: fd  	port map(d => mux_p(4),  c => clkx3p5not, q => dataout(4)) ;
fd_mnn5	: fd  	port map(d => clkcn,     c => clkx3p5not, q => clkout(0)) ;
	
clkcp_d <= '1' when pba = "001" or pba = "010" or pba = "101" else '0' ;	-- Use these two lines for 3:4 output clock
clkcn_d <= '1' when pba = "001" or pba = "100" or pba = "101" else '0' ;

--clkcp_d <= '1' when pba = "001" or pba = "010" or pba = "101" or pba = "110" else '0' ;	-- Use these two lines for 4:3 output clock
--clkcn_d <= '1' when pba = "001" or pba = "010" or pba = "101" or pba = "100" else '0' ;

fd_clkcp   : fd port map (c => clkx3p5,  d => clkcp_d,  q => clkout(1));
fd_clkcn   : fd port map (c => clkx3p5,  d => clkcn_d,  q => clkcn);

xra <= clkd2d_a xor clkd2d_b ;
fd_xra	: fdc port map (c => clkx3p5, d => xra,  clr => rst, q => xrareg);
             
phase_bna <= "110" when xrareg = '1' else
             "000" when pba = "110" else
             pba + 1 ;

phase_bnb <= "110" when xrareg = '1' else
             "000" when pbb = "110" else
             pbb + 1 ;
             
notclk3p5d2 <= not clk3p5d2a ;			-- divide clk35 by 2
fd_c35d2a : fdc port map (c => clkx3p5,  d => notclk3p5d2, clr => rst, q => clk3p5d2a);
fd_c35d2b : fdc port map (c => clkx3p5,  d => notclk3p5d2, clr => rst, q => clk3p5d2b);

fd_cb   : fdc port map (c => clk,      d => clk3p5d2b, clr => rst, q => clkd2);
fd_cbda : fdc port map (c => clkx3p5,  d => clkd2,     clr => rst, q => clkd2d_a);
fd_cbdb : fdc port map (c => clkx3p5,  d => clkd2d_a,  clr => rst, q => clkd2d_b);

fdcpba0 : fdc port map (c => clkx3p5, d => phase_bna(0), clr => rst, q => pba(0));
fdcpba1 : fdc port map (c => clkx3p5, d => phase_bna(1), clr => rst, q => pba(1));
fdcpba2 : fdc port map (c => clkx3p5, d => phase_bna(2), clr => rst, q => pba(2));
fdcpba3 : fdc port map (c => clkx3p5, d => pba(1),       clr => rst, q => pbad(1));
fdcpba4 : fdc port map (c => clkx3p5, d => pba(2),       clr => rst, q => pbad(2));
fdcpba5 : fdc port map (c => clkx3p5, d => pbad(2),      clr => rst, q => pbadd(2));

fdcpbb0 : fdc port map (c => clkx3p5, d => phase_bnb(0), clr => rst, q => pbb(0));
fdcpbb1 : fdc port map (c => clkx3p5, d => phase_bnb(1), clr => rst, q => pbb(1));
fdcpbb2 : fdc port map (c => clkx3p5, d => phase_bnb(2), clr => rst, q => pbb(2));

ce0_d <= '1' when pbb = "001" or pbb = "100" else '0' ;
ce1_d <= '1' when pbb = "100" or pbb = "000" else '0' ;

fd_ce0 : fd port map (c => clkx3p5, d => ce0_d, q => ce0);
fd_ce1 : fd port map (c => clkx3p5, d => ce1_d, q => ce1);

fd_db0  : fde port map (c => clkx3p5, d => dataint(0),  ce => ce0, q => db(0)) ;
fd_db1  : fde port map (c => clkx3p5, d => dataint(1),  ce => ce0, q => db(1)) ;
fd_db2  : fde port map (c => clkx3p5, d => dataint(2),  ce => ce0, q => db(2)) ;
fd_db3  : fde port map (c => clkx3p5, d => dataint(3),  ce => ce0, q => db(3)) ;
fd_db4  : fde port map (c => clkx3p5, d => dataint(4),  ce => ce0, q => db(4)) ;
fd_db5  : fde port map (c => clkx3p5, d => dataint(5),  ce => ce0, q => db(5)) ;
fd_db6  : fde port map (c => clkx3p5, d => dataint(6),  ce => ce0, q => db(6)) ;
fd_db7  : fde port map (c => clkx3p5, d => dataint(7),  ce => ce0, q => db(7)) ;
fd_db8  : fde port map (c => clkx3p5, d => dataint(8),  ce => ce0, q => db(8)) ;
fd_db9  : fde port map (c => clkx3p5, d => dataint(9),  ce => ce0, q => db(9)) ;
fd_db10 : fde port map (c => clkx3p5, d => dataint(10), ce => ce0, q => db(10)) ;
fd_db11 : fde port map (c => clkx3p5, d => dataint(11), ce => ce0, q => db(11)) ;
fd_db12 : fde port map (c => clkx3p5, d => dataint(12), ce => ce0, q => db(12)) ;
fd_db13 : fde port map (c => clkx3p5, d => dataint(13), ce => ce0, q => db(13)) ;
fd_db14 : fde port map (c => clkx3p5, d => dataint(14), ce => ce0, q => db(14));
fd_db15 : fde port map (c => clkx3p5, d => dataint(15), ce => ce0, q => db(15));
fd_db16 : fde port map (c => clkx3p5, d => dataint(16), ce => ce0, q => db(16));
fd_db17 : fde port map (c => clkx3p5, d => dataint(17), ce => ce0, q => db(17));
fd_db18 : fde port map (c => clkx3p5, d => dataint(18), ce => ce0, q => db(18));
fd_db19 : fde port map (c => clkx3p5, d => dataint(19), ce => ce0, q => db(19));
fd_db20 : fde port map (c => clkx3p5, d => dataint(20), ce => ce0, q => db(20));
fd_db21 : fde port map (c => clkx3p5, d => dataint(21), ce => ce0, q => db(21));
fd_db22 : fde port map (c => clkx3p5, d => dataint(22), ce => ce0, q => db(22));
fd_db23 : fde port map (c => clkx3p5, d => dataint(23), ce => ce0, q => db(23));
fd_db24 : fde port map (c => clkx3p5, d => dataint(24), ce => ce0, q => db(24));
fd_db25 : fde port map (c => clkx3p5, d => datainr(25), ce => ce1, q => db(25));
fd_db26 : fde port map (c => clkx3p5, d => datainr(26), ce => ce1, q => db(26));
fd_db27 : fde port map (c => clkx3p5, d => datainr(27), ce => ce1, q => db(27));
fd_db28 : fde port map (c => clkx3p5, d => datainr(28), ce => ce1, q => db(28));
fd_db29 : fde port map (c => clkx3p5, d => datainr(29), ce => ce1, q => db(29));
fd_db30 : fde port map (c => clkx3p5, d => datainr(30), ce => ce1, q => db(30));
fd_db31 : fde port map (c => clkx3p5, d => datainr(31), ce => ce1, q => db(31));
fd_db32 : fde port map (c => clkx3p5, d => datainr(32), ce => ce1, q => db(32));
fd_db33 : fde port map (c => clkx3p5, d => datainr(33), ce => ce1, q => db(33));
fd_db34 : fde port map (c => clkx3p5, d => datainr(34), ce => ce1, q => db(34));

fd_d25 	: fd port map (c => clk, d => dataint(25), q => datainr(25));
fd_d26 	: fd port map (c => clk, d => dataint(26), q => datainr(26));
fd_d27 	: fd port map (c => clk, d => dataint(27), q => datainr(27));
fd_d28 	: fd port map (c => clk, d => dataint(28), q => datainr(28));
fd_d29 	: fd port map (c => clk, d => dataint(29), q => datainr(29));
fd_d30 	: fd port map (c => clk, d => dataint(30), q => datainr(30));
fd_d31 	: fd port map (c => clk, d => dataint(31), q => datainr(31));
fd_d32 	: fd port map (c => clk, d => dataint(32), q => datainr(32));
fd_d33 	: fd port map (c => clk, d => dataint(33), q => datainr(33));
fd_d34 	: fd port map (c => clk, d => dataint(34), q => datainr(34));
	
end arch_serdes_5b_7to1 ;

